"""Copyright (c) 2023, Aydin Abdi.

This module is analyze and extracts the data from
test results and can visualize the data in different ways.
"""
from qa_analytics_insights._version import version

__author__ = "Aydin Abdi"
__license__ = "MIT"
__version__ = version
